/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.exception;

import org.lwsdo.util.I18NUtils;

public class AmbiguousMatchException extends ApplicationException {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private String propertyPath1;
	private String propertyPath2;	
	
	public AmbiguousMatchException() {
		super();
	}
	
	public AmbiguousMatchException(String propertyPath1, String propertyPath2, Exception e) {
		super(getObjectNotFoundMessage(propertyPath1, propertyPath2), e);
		this.setPropertyPath1(propertyPath1);
		this.setPropertyPath2(propertyPath2);		
	}
	
	public AmbiguousMatchException(String propertyPath1, String propertyPath2) {
		super(getObjectNotFoundMessage(propertyPath1, propertyPath2));
		this.setPropertyPath1(propertyPath1);
		this.setPropertyPath2(propertyPath2);		
	}
	
	public AmbiguousMatchException(Throwable t) {
		super(t);
	}
	
	private static String getObjectNotFoundMessage (String propertyPath1, String propertyPath2 ) {
		String[] params = new String[2];
		params[0] = propertyPath1;
		params[1] = propertyPath2;
		return  I18NUtils.getResource(  "exception.ambiguousMatch",I18NUtils.CORE_RESOURCES, params);		
	}

	public String getPropertyPath2() {
		return propertyPath2;
	}

	public void setPropertyPath2(String propertyPath2) {
		this.propertyPath2 = propertyPath2;
	}

	public String getPropertyPath1() {
		return propertyPath1;
	}

	public void setPropertyPath1(String propertyPath1) {
		this.propertyPath1 = propertyPath1;
	}

}
