/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Map;

import org.lwsdo.util.ObjectCreator;

public class DataObjectList {

	private ExtendedDataObject dataObject;
	private Property property;

	public List<ExtendedDataObject> list(CallInfo callInfo) {
		Object list = (property != null) ? ((ExtendedProperty)property).getValue(dataObject) : dataObject.getInstance();
		if(list == null)
			return new ArrayList();

		Collection<Object> collection = null;
		if(Collection.class.isAssignableFrom(list.getClass()))
			collection = (Collection<Object>) list;
		else if(Map.class.isAssignableFrom(list.getClass()))
			collection = ((Map<Object, Object>) list).values();
		else
			//throw new IllegalArgumentException("Cannot create a DataObjectList for a non-list property: " + property.getName() + ", with type: " + property.getType().getName());
			return new ArrayList();

		Type type = (property != null) ? property.getType() : ((ExtendedProperty)dataObject.getContainmentProperty()).getElementType();		
		List<ExtendedDataObject> result = new ArrayList<ExtendedDataObject>();
		ObjectCreator objectCreator = dataObject.getObjectCreator();
		for(Object element: collection) {
			if(callInfo == null)
				result.add(objectCreator.createDataObject(element, type, (ExtendedDataObject) dataObject.getContainer(), dataObject.getContainmentProperty()));
			else {
				CallInfo child = new CallInfo(null, null, callInfo, null);
				result.add((ExtendedDataObject) objectCreator.createTarget(child, element, null));
			}
		}

		return result;
	}

	public DataObjectList(ExtendedDataObject dataObject, Property property) {
		this.dataObject = dataObject;
		this.property = property;
	}

	/**
	 * This constructor is for DataObject representing a collection
	 * @param dataObject
	 */
	public DataObjectList(ExtendedDataObject dataObject) {
		this.dataObject = dataObject;
	}	
}
