/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model;

import java.util.Set;

import org.lwsdo.model.DependencyGraph.Edge;

public interface Node {
	
	public boolean isVisited();
	
	public void setVisited(boolean visited);	

	public Set<Edge> getOutgoing();
	
	public Set<Edge> getIncoming();	
	
	public Object getUnlinkValue();

	public Object getLinkValue();

	public Object getInstance();

	public void addIncomingEdge(Edge edge);

	public void removeIncomingEdge(Edge edge);

	public void addOutgoingEdge(Edge edge);

	public void removeOutgoingEdge(Edge edge);

	public void unlink();

	public Edge getNewEdge(boolean isCollection);

	public void markCascadeTree();

	public void replaceInstance(Object newInstance);
	
	public boolean isCollectionNode();	

	public Edge addIncomingEdge(Node from, CallInfo ci);
	
	public Edge getOwner();

	public boolean removeCollectionObjects(CallInfo callInfo, ExtendedDataObject targetCollection, ExtendedDataObject sourceCollection) throws Exception;

	boolean isPersistent();

	boolean isModified();

	void setModified(boolean modified);
}