/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model;

import java.util.ArrayList;
import java.util.List;

/**
 * SimpleType have no properties
 * 
 * @author Dilip Dalton
 * 
 */
public class SimpleType implements BasicType {

	private Class<?> instanceClass;
	private ClassResolver classResolver;	

	public SimpleType(Class<?> clazz) {
		this.instanceClass = clazz;
		classResolver = new ClassResolver(this);
	}

	@Override
	public String getName() {
		// The name is unique in the namespace because if is qualified by the
		// package name
		return instanceClass.getName();
	}

	@Override
	public String getURI() {
		return null;
	}

	@Override
	public Class<?> getInstanceClass() {
		return instanceClass;
	}

	@Override
	public boolean isInstance(Object object) {
		return getClass().isAssignableFrom(object.getClass());
	}

	/**
	 * The property names within the type are unique
	 */
	@Override
	public List<Property> getProperties() {
		return new ArrayList<Property>();
	}

	@Override
	public Property getProperty(String propertyName) {
		return null;
	}

	@Override
	public boolean isDataType() {
		return true;
	}

	@Override
	public boolean isOpen() {
		return false;
	}

	@Override
	public boolean isSequenced() {
		return false;
	}

	@Override
	public boolean isAbstract() {
		return false;
	}

	@Override
	public List<Type> getBaseTypes() {
		return new ArrayList<Type>();
	}

	@Override
	public List<Property> getDeclaredProperties() {
		return new ArrayList<Property>();
	}

	@Override
	public List<?> getAliasNames() {
		return new ArrayList<String>();
	}

	@Override
	public List<?> getInstanceProperties() {
		return new ArrayList<Object>();
	}

	@Override
	public Object get(Property property) {
		return null;
	}

	@Override
	public ClassResolver getClassResolver() {
		return classResolver;
	}
}
