/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model;

public class VOTypeMapper extends AbstractTypeMapper {
	private Package referencePackage;
	private Package derivedPackage;

	public static String VO_SUFFIX = "VO";
	
	public void setReferencePackage(String packageName) {
		this.referencePackage = Package.getPackage(packageName);
	}
	
	public void setDerivedPackage(String packageName) {
		this.derivedPackage = Package.getPackage(packageName);
	}	

	/* (non-Javadoc)
	 * @see org.LWSDO.model.TypeMapper#toReference(java.lang.Class)
	 */
	@Override
	public Class<?> toReference(Class<?> derivedClass) {
		String fullClassName = derivedClass.getCanonicalName();
		// Change to the reference package
		String toClassName = fullClassName.replaceFirst(derivedPackage.getName(), referencePackage.getName()); 

		// Remove the VO_SUFFIX
		int toNameLen = toClassName.length();
		StringBuffer toNameBuf = (new StringBuffer(toClassName)).delete(
				toNameLen - VO_SUFFIX.length(), toNameLen); 
		toClassName = toNameBuf.toString();		

		try {
			return Class.forName(toClassName);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}	

		return null;
	}

	/* (non-Javadoc)
	 * @see org.LWSDO.model.TypeMapper#toDerived(java.lang.Class)
	 */
	@Override
	public Class<?> toDerived(Class<?> referenceClass) {
		String fullClassName = referenceClass.getCanonicalName();
		// Change to the derived package
		String toClassName = fullClassName.replaceFirst(referencePackage.getName(), derivedPackage.getName()); 

		// Append the VO_SUFFIX
		toClassName += VO_SUFFIX;

		try {
			return Class.forName(toClassName);
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
		}	

		return null;		
	}

	public Class<?> getClass(Class<?> clazz) {
		Class<?> result = null;

		switch(getDirection()) {
		case DERIVEDTOREFERENCE:
			result = toReference(clazz);
			break;
		case REFERENCETODERIVED:
			result = toDerived(clazz);
			break;
		case REFERENCETOREFERENCE:
		case DERIVEDTODERIVED:
		default:
			result = clazz;
			break;
		}

		return result;
	}

	@Override
	public TypeMapper newInstance(MapperDirection direction) {
		TypeMapper mapper = new VOTypeMapper();
		mapper.setDirection(direction);
		
		return mapper;		
	}	
}
