/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model.action;

import java.io.Serializable;

import javax.persistence.metamodel.Attribute.PersistentAttributeType;

import org.lwsdo.model.ExtendedDataObject;
import org.lwsdo.model.ExtendedProperty;
import org.lwsdo.model.Property;

/**
 * Uniquely identifies a persistent entity.
 */
public final class PropertyKey implements Serializable {
	private static final long serialVersionUID = 1L;
	
	private final ExtendedDataObject dataObject;
	private final Property property;
	private final PersistentAttributeType associationType;
	
	public Property getProperty() {
		return property;
	}

	public ExtendedDataObject getDataObject() {
		return dataObject;
	}	

	public PropertyKey(ExtendedDataObject dataObject, Property property) {
		this.dataObject = dataObject; 
		this.property = property;
		
		this.associationType = ((ExtendedProperty)property).getAssociationType();
	}

	@Override
	public int hashCode() {
		int result = 17;
		result = 37 * result + System.identityHashCode(dataObject);
		result = 37 * result + System.identityHashCode(property);
		return result;
	}

	@Override
	public boolean equals(Object other) {
		PropertyKey otherKey = (PropertyKey) other;
		return otherKey.dataObject == this.dataObject && otherKey.property == this.property;
	}	

	public boolean isOneToOne() {
		return associationType == PersistentAttributeType.ONE_TO_ONE;
	}
}
