/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model.action;

import org.lwsdo.model.ExtendedDataObject;
import org.lwsdo.model.ExtendedProperty;
import org.lwsdo.model.Settings;


public final class SetterAction implements Executable {

	private final Object   value;
	private final PropertyKey key;
	private final Executable triggeringAction;

	public SetterAction(Object value, PropertyKey key, Executable triggeringAction) {
		this.value = value;
		this.key = key;
		this.triggeringAction = triggeringAction;
	}

	@Override
	public void execute() {
		setValue(value);
	}
	
	public void setValue(Object value) {
		ExtendedProperty prop = ((ExtendedProperty)key.getProperty());
		
		ExtendedDataObject invokeOn = key.getDataObject();
		// First check if this property is part of this data object
		if(ExtendedDataObject.class.isAssignableFrom(invokeOn.getClass())) {
			if(((ExtendedDataObject)invokeOn).getType().getProperty(prop.getName()) == null && prop.getMapPath() != null && prop.getMapPath().contains(Settings.PATH_DELIMITER)) {
				// Get the embedded object
				invokeOn = (ExtendedDataObject) ((ExtendedDataObject)invokeOn).get( prop.getMapPath().substring(0, prop.getMapPath().lastIndexOf(Settings.PATH_DELIMITER)));
			}
		}
		
		prop.setValue(invokeOn, value);
	}		

	@Override
	public PropertyKey getKey() {
		return key;
	}

	@Override
	public Executable getTriggeringAction() {
		return triggeringAction;
	}	

	public Object getValue() {
		return value;
	}		
}
