/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model.operation;

import org.lwsdo.model.CallInfo;
import org.lwsdo.model.ExtendedDataObject;
import org.lwsdo.model.Node;
import org.lwsdo.util.ClassUtil;

public class CloneOperation extends AbstractOperation {
	
	private Object result;

	@Override
	protected void processCollection(CallInfo callInfo) throws Exception {

		if(callInfo.isDataType())
			return;

		for (Object nextSource : ((ExtendedDataObject)callInfo.getSource()).getList(callInfo)) {
			CallInfo next = new CallInfo(nextSource, null, callInfo, null);
			if(next.isCascadable()) {
				next.setTarget(getExistingTarget(next));
				next.setTarget(createTarget(next, null));
				processValue(next);	
				callInfo.getTargetRoot().getDependencyGraph().addEdge(null, next, (Node) next.getTarget());				
			} else
				next.setTarget(createTarget(next, ClassUtil.getInstance(nextSource), null));
		}
		cloneToMany(callInfo);
	}

	@Override
	protected void setResult(Object target) {
		this.result = target;
	}	
	
	@Override
	public Object getResult() {
		return result;
	}	
}
