/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model.operation;

import org.lwsdo.model.CallInfo;


public interface Operation {
	/**
	 * The logic in the copying of data from the input to the output objects
	 * @param callInfo
	 * @throws Exception 
	 */
	public void execute(CallInfo callInfo);

	/**
	 * Allow any pre-processing before the operation occurs between the source and target 
	 * @param callInfo
	 */
	public boolean preCreatePropertyCallback(CallInfo callInfo);

	/**
	 * Allows operation specific behavior in the creation of DataObjects
	 * @param ci
	 * @param desiredClass
	 * @return
	 */
	public Object createTarget(CallInfo ci, Class<?> desiredClass);

	/**
	 * Allows a new DataObject to be created based on an existing target instance object
	 * @param ci
	 * @param targetInstance
	 * @param desiredClass
	 * @return
	 */
	public Object createTarget(CallInfo ci, Object targetInstance, Class<?> desiredClass);
	
	/**
	 * Returns the result of the execution if any.
	 * @return
	 */
	public Object getResult();
}
