/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model.view;

import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlTransient;
import javax.xml.bind.annotation.XmlValue;

import org.lwsdo.model.view.expression.AbstractFunctionExpression;
import org.lwsdo.model.view.expression.ExpressionFactory;

@XmlAccessorType(XmlAccessType.FIELD)
public class Filter {

	@XmlValue
	protected String expression;
	
	@XmlTransient
	protected AbstractFunctionExpression functionExpression;

	public Filter narrow() {
		Filter result = new Filter();
		result.expression = ExpressionFactory.getFilterFunction(expression);

		return result;
	}

	public String getExpression() {
		return expression;
	}

	public void setExpression(String expression) {
		this.expression = expression;
	}

	public String getAttribute() {
		return functionExpression.getAttributeName();
	}

	public void normalize(Map<String, String> normalizedNames) {
		functionExpression.normalize(normalizedNames);
	}	
	
	public void init(Set<String> parameters) {
		functionExpression.initIncluded(parameters);
	}
}
