/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model.view.expression;

import java.util.Map;
import java.util.Set;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.lwsdo.util.ClassUtil;

public abstract class AbstractFunctionExpression implements Expression {

	private String expression;

	protected boolean included;  // Specifies if this filter should be used in the query. This flag can be set to false if
	// the expression refers to a parameter that is not set

	protected abstract String getAttributePattern();

	public String getExpression() {
		return expression;
	}

	public void setExpression(String value) {
		this.expression = value;
	}

	public boolean isIncluded() {
		return included;
	}

	public void setIncluded(boolean included) {
		this.included = included;
	}	

	public AbstractFunctionExpression copy() {
		AbstractFunctionExpression result;
		try {
			result = this.getClass().newInstance();
		} catch (Exception e) {
			throw ClassUtil.wrapRun(e);
		}
		result.setExpression(this.getExpression());
		result.setIncluded(this.isIncluded());

		return result;
	}

	public String getAttributeName() {
		// An enhancement would be to return a set of attributes if an expression consists of multiple filter functions			
		Pattern pattern = Pattern.compile( getAttributePattern() ); 
		Matcher matcher = pattern.matcher(getExpression());
		matcher.find();

		if(matcher.matches() && matcher.groupCount() > 1)
			return matcher.group(2);
		else
			return null;
	}

	public String getParameterName() {
		Pattern pattern = Pattern.compile( getAttributePattern() ); 
		Matcher matcher = pattern.matcher(expression);
		matcher.find();	

		if(matcher.matches() && matcher.groupCount() > 2)
			return matcher.group(3);
		else
			return null;		
	}	

	public void normalize(Map<String, String> normalizedNames) {	
		
		Pattern pattern = Pattern.compile( getAttributePattern() ); 
		Matcher matcher = pattern.matcher(expression);
		matcher.find();	

		StringBuffer result = new StringBuffer(matcher.group(1));
		result.append(normalizedNames.get(getAttributeName())); // Second group

		for(int groupNo = 3; groupNo <= matcher.groupCount(); groupNo++)
			result.append(matcher.group(groupNo));

		expression = result.toString();
	}
	
	public void initIncluded(Set<String> parameterNames) {
		if(getParameterName() == null || parameterNames.contains(getParameterName()))
			included = true;
	}
}
