/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.model.view.expression;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.lwsdo.util.ClassUtil;


public class ExpressionFactory {
	private static final String FUNCTION_NAME_PAT = "^(\\w+)\\s*\\("; // ^(\w+)\s*\( - Get the name of the filter function

	private static final Map<String, Class> FUNCTION_MAP  = new HashMap<String, Class>() {
		{
			put("ILIKE", IlikeFunctionExpression.class);
			put("IN",    InFunctionExpression.class);
			put("EQUAL", EqualFunctionExpression.class);
			put("GE",    GeFunctionExpression.class);
			put("ASC",   AscFunctionExpression.class);			
		}
	};

	protected AbstractFunctionExpression getExpression(String expression) {
		Class clazz = FUNCTION_MAP.get(getFilterFunction(expression).toUpperCase());
		AbstractFunctionExpression result;
		try {
			result = (AbstractFunctionExpression) clazz.newInstance();
		} catch (Exception e) {
			throw ClassUtil.wrapRun(e);
		}
		result.setExpression(expression);

		return result;
	}	

	public static String getFilterFunction(String expression) {
		Pattern pattern = Pattern.compile(FUNCTION_NAME_PAT); 
		Matcher matcher = pattern.matcher(expression);
		matcher.find();

		if(matcher.matches())
			return matcher.group(2);
		else
			throw new RuntimeException("Cannot find the filter function");

	}			
}
