/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.service;

import java.util.ArrayList;
import java.util.List;

import org.lwsdo.model.AggregateAction;
import org.lwsdo.model.ExtendedDataObject;
import org.lwsdo.model.ExtendedType;
import org.lwsdo.model.MapperDirection;
import org.lwsdo.model.Settings;
import org.lwsdo.util.ObjectCreator;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Component;

@Component
@Scope(value = "prototype")
public class AggregateService {

	@Autowired
	protected SpringDASFactory springDASFactory;	

	protected DataAccessService getDAS() {
		DataAccessService das = springDASFactory.create(); // TODO: For now we create this object for every call, this can later be optimized

		return das;
	}

	public Object clone(Object entity, Class<?> desiredSubClass) {
		DataAccessService das = getDAS();

		ObjectCreator oc = new ObjectCreator(das, MapperDirection.REFERENCETOREFERENCE);
		ExtendedDataObject from = oc.createDataObject(entity, (ExtendedType) das.getType(entity.getClass()), null, null);
		ExtendedDataObject to = (ExtendedDataObject) from.clone(new Settings());		

		return to.getInstance();
	}

	public void delete(Object entity) {
		// TODO: 
	}	

	public List<?> query(Object entity, Settings settings) {
		DataAccessService das = getDAS();
		if(settings.doFlush())
			das.getPersistenceOrchestrator().flush();

		ObjectCreator oc = new ObjectCreator(das, MapperDirection.REFERENCETODERIVED);
		ExtendedDataObject from = oc.createDataObject(entity, (ExtendedType) das.getType(entity.getClass()), null, null);		
		
		List<?> dataObjects = from.query(settings);
		List<Object> result = new ArrayList<Object>();
		for(Object obj: dataObjects) 
			result.add( ((ExtendedDataObject)obj).getInstance() );
		
		return result;
	}

	public Object read(Object entity, Settings settings) {
		DataAccessService das = getDAS();

		ObjectCreator oc = new ObjectCreator(das, MapperDirection.REFERENCETODERIVED);
		ExtendedDataObject from = oc.createDataObject(entity, (ExtendedType) das.getType(entity.getClass()), null, null);
		ExtendedDataObject to = (ExtendedDataObject) from.read(settings);		

		return to.getInstance();
	} 	

	public Object create(Object entity, Settings settings) {
		DataAccessService das = getDAS();

		ObjectCreator oc = new ObjectCreator(das, MapperDirection.DERIVEDTOREFERENCE);
		ExtendedDataObject from = oc.createDataObject(entity, (ExtendedType) das.getType(entity.getClass()), null, null);
		ExtendedDataObject to = (ExtendedDataObject) from.create(settings);					

		return to.getInstance();		
	}

	public Object update(Object entity, Settings settings) {
		DataAccessService das = getDAS();

		if(settings.getAction() != AggregateAction.MERGE && settings.getAction() != AggregateAction.UPDATE)
			throw new IllegalStateException("The default action should either by UPDATE or MERGE");

		ObjectCreator oc = new ObjectCreator(das, MapperDirection.DERIVEDTOREFERENCE);
		ExtendedDataObject from = oc.createDataObject(entity, (ExtendedType) das.getType(entity.getClass()), null, null);
		ExtendedDataObject to = (ExtendedDataObject) from.update(settings);					

		return to.getInstance();
	}	

}
