/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.service;

import java.io.Serializable;
import java.util.Collection;
import java.util.Collections;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;

public class HibernateDAOTemplate<T> implements DAOTemplate<T> {
    protected Class<? extends T> persistentClass;
    
    @Autowired
    private SessionFactory sessionFactory;    
    
    protected Session getSession() {
        if (this.sessionFactory.getCurrentSession() == null)
            throw new IllegalStateException("Session has not been set on DAO before usage");
        return this.sessionFactory.getCurrentSession();
    }       
	
	@SuppressWarnings("unchecked")
	@Override
	public T findById(Object id) {
        T entity;
        entity = (T) getSession().load(persistentClass, (Serializable) id);
 
        return entity;
	}

	@SuppressWarnings("unchecked")
	@Override
	public T findByName(String name) {
        Criteria crit = getSession().createCriteria(persistentClass);
        crit.add(Restrictions.eq("name", name));
		List<T> names = crit.list();
		if (names != null && !names.isEmpty()) {
			return names.get(0);
		}
		return null;
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<T> findByIds(Collection ids) {
        if (ids.isEmpty()) {
            return Collections.emptyList();
        }
        Criteria crit = getSession().createCriteria(persistentClass);
        crit.add(Restrictions.in("id", ids));        
        return crit.list();
	}

	@SuppressWarnings("unchecked")
	@Override
	public List<T> findAll() {
        Criteria crit = getSession().createCriteria(persistentClass);
        return crit.list();
	}

	@Override
	public T saveOrUpdate(T entity) {
        getSession().saveOrUpdate(entity);
        return entity;
	}

	@Override
	public void delete(T entity) {
        getSession().delete(entity);
		
	}

	@Override
	public void refresh(T entity) {
    	if (entity != null && this.getSession() != null) {
    		this.getSession().refresh(entity);
    	}
	}

	@Override
	public void clear() {
    	if (this.getSession() != null) {
    		this.getSession().clear();
    	}
	}

	@Override
	public void setPersistentClass(T clazz) {
		this.persistentClass = (Class<? extends T>) clazz;
	}

}
