/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.util;

import java.text.MessageFormat;
import java.util.Locale;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

public class I18NUtils {
    /**
     * Constants for the resource bundle names
     */
    public static final String CORE_RESOURCES = "CoreResources";
  
    
    /**
     * Returns the localized string with the given key from the given bundle.
     * Works with messsage with no parameter.
     * @param key
     * @param bundleName
     * @param locale
     * @return the localized resource
     */
    public static String getResource(String key, String bundleName, Locale locale) {
        try {
            ResourceBundle bundle = ResourceBundle.getBundle(bundleName, locale);
            return bundle.getString(key);
        } catch (MissingResourceException e) {
            // mimic the way jspc returns when resource is missing
            return "??? " + key + " ???";
        }
    }  
    
    public static String getResource(String key, String bundleName, Locale locale, String[] params ) {
        return MessageFormat.format(getResource(key, bundleName, locale), (Object[])params);
    }
    
      
    public static String getResource(String key, String bundleName) { 
    	return getResource(key, bundleName, Locale.getDefault());
    }   
     
    public static String getResource(String key, String bundleName, String[] params ) {
    	return getResource(key, bundleName, Locale.getDefault(), params);
    }   
  
}