/**
 * LWSDO, a high performance Domain Driven Design framework.
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * LWSDO is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package org.lwsdo.util;

import java.util.Arrays;
import java.util.List;

import org.lwsdo.model.DefaultTypeMapper;
import org.lwsdo.model.TypeMapper;
import org.lwsdo.model.VOTypeMapper;
import org.springframework.stereotype.Component;

@Component
public class LwsdoConfiguration {
	private Class<? extends TypeMapper>[] typeMappers;
	private TypeMapper mappingStrategy;
	private AccessType accessType;
	
	public LwsdoConfiguration() {
		typeMappers = (Class<? extends TypeMapper>[]) new Class<?>[2];
		typeMappers[0] = DefaultTypeMapper.class;
		typeMappers[1] = VOTypeMapper.class;
		
		setMappingStrategy(new DefaultTypeMapper());
	}

	public Class<?>[] getTypeMappers() {
		return typeMappers;
	}

	public void setTypeMappers(Class<?>[] mappers) {
		List<Class<?>> list = Arrays.asList(mappers);
		for(Class<?> builtIn : list) {
			if(!list.contains(builtIn))
				list.add(builtIn);
		}
		this.typeMappers = list.toArray(this.typeMappers);
	}

	public TypeMapper getMappingStrategy() {
		return mappingStrategy;
	}

	public void setMappingStrategy(TypeMapper mappingStrategy) {
		this.mappingStrategy = mappingStrategy;
	}

	public AccessType getAccessType() {
		return accessType;
	}

	public void setAccessType(AccessType accessType) {
		this.accessType = accessType;
	}

}